import types

LAMP_1RG = 0x0000
LAMP_2RG = 0x0001
LAMP_3RG = 0x0002
LAMP_4RG = 0x0003
LAMP_5RG = 0x0004

MODE_CONTINUOUS = 0x00
MODE_SWITCHING_1HZ = 0x01
MODE_SWITCHING_2HZ = 0x02

COLOR_NONE = 0x00
COLOR_GREEN = 0x01
COLOR_RED = 0x02
COLOR_YELLOW = 0x03

COLOR_GREEN_RED = 0x12
COLOR_GREEN_YELLOW = 0x13
COLOR_RED_YELLOW = 0x23


class LampControl:
    _ACK_TOUT_SEC = 1

    def __init__(self, modbus_transport, lamp_address):
        if lamp_address not in [LAMP_1RG, LAMP_2RG, LAMP_3RG, LAMP_4RG, LAMP_5RG]:
            raise ValueError("Invalid lamp_address")

        self._modbus_transport = modbus_transport
        self._lamp_address = lamp_address

    def set_state(self, mode, color):
        """
        :param mode: One of the MODE_XXX constants
        :type mode: int
        :type color: int
        :rtype: int
        """
        if mode not in [MODE_CONTINUOUS, MODE_SWITCHING_1HZ, MODE_SWITCHING_2HZ]:
            return types.RC_FAIL

        if color not in [COLOR_NONE, COLOR_GREEN, COLOR_RED, COLOR_YELLOW, COLOR_YELLOW,
                         COLOR_GREEN_RED, COLOR_GREEN_YELLOW, COLOR_RED_YELLOW]:
            return types.RC_FAIL

        retcode = self._modbus_transport.write_reg(self._lamp_address, (mode << 8) + color, LampControl._ACK_TOUT_SEC)
        return retcode

