import types

HEADER_1_4S2M__MOTOR1 = 0x0011
HEADER_1_4S2M__MOTOR2 = 0x0012
HEADER_1_4S2M__MOTOR1_FAST = 0x0031
HEADER_1_4S2M__MOTOR2_FAST = 0x0032
HEADER_2_4S2M__MOTOR1 = 0x0021
HEADER_2_4S2M__MOTOR2 = 0x0022
HEADER_2_4S2M__MOTOR1_FAST = 0x0041
HEADER_2_4S2M__MOTOR2_FAST = 0x0042

MIN_STEP_COUNT = 0
MAX_STEP_COUNT0 = 255
MAX_STEP_COUNT1 = 0x7FFF

FIRMWARE_VER_REG = 0xFFFF

class CheckVer:
    _ACK_TOUT_SEC = 0.25

    def __init__(self, modbus_transport):
        self._modbus_transport = modbus_transport

    def check_ver(self, res):
        self._modbus_transport.read_reg(FIRMWARE_VER_REG, res, CheckVer._ACK_TOUT_SEC)


class MotorControl:
    _CLOCKWISE = 0x00
    _ANTICLOCKWISE = 0x100
    _ACK_TOUT_SEC = 5

    def __init__(self, modbus_transport, motor_address, v=0):
        if v > 1:
            if motor_address not in [HEADER_1_4S2M__MOTOR1, HEADER_1_4S2M__MOTOR2,
                                     HEADER_2_4S2M__MOTOR1, HEADER_2_4S2M__MOTOR2,
                                     HEADER_1_4S2M__MOTOR1_FAST, HEADER_1_4S2M__MOTOR2_FAST,
                                     HEADER_2_4S2M__MOTOR1_FAST, HEADER_2_4S2M__MOTOR2_FAST]:
                raise ValueError("Invalid motor_address")
        else:
            if motor_address not in [HEADER_1_4S2M__MOTOR1, HEADER_1_4S2M__MOTOR2,
                                     HEADER_2_4S2M__MOTOR1, HEADER_2_4S2M__MOTOR2]:
                raise ValueError("Invalid motor_address")

        self._modbus_transport = modbus_transport
        self._motor_address = motor_address

    @staticmethod
    def _validate_step_count(step_count, v):
        if v > 1:
            return MIN_STEP_COUNT <= step_count <= MAX_STEP_COUNT1
        else:
            return MIN_STEP_COUNT <= step_count <= MAX_STEP_COUNT0

    def rotate_cw(self, step_count, v=0):
        """
        :type step_count: int
        :rtype: int
        """
        if not MotorControl._validate_step_count(step_count, v):
            print "the specified steps number is not supported by fw version ", v
            return types.RC_FAIL

        if self._motor_address in [HEADER_1_4S2M__MOTOR1_FAST, HEADER_1_4S2M__MOTOR2_FAST,
                                   HEADER_2_4S2M__MOTOR1_FAST, HEADER_2_4S2M__MOTOR2_FAST]:
            tout = 0.25 + step_count * 0.0027
        else:
            tout = 0.25 + 6 * step_count * 0.0027

        #tout = MotorControl._ACK_TOUT_SEC
        if v > 1:
            retcode = self._modbus_transport.write_reg(
                self._motor_address, MotorControl._CLOCKWISE + int(int(step_count / 256) * 512) + (step_count % 256), tout)
        else:
            retcode = self._modbus_transport.write_reg(
                self._motor_address, MotorControl._CLOCKWISE + (step_count % 256), tout)
        return retcode

    def rotate_ccw(self, step_count, v=0):
        """
        :type step_count: int
        :rtype: int
        """
        if not MotorControl._validate_step_count(step_count, v):
            return types.RC_FAIL

        if self._motor_address in [HEADER_1_4S2M__MOTOR1_FAST, HEADER_1_4S2M__MOTOR2_FAST,
                                   HEADER_2_4S2M__MOTOR1_FAST, HEADER_2_4S2M__MOTOR2_FAST]:
            tout = 0.25 + step_count * 0.0027
        else:
            tout = 0.25 + 6 * step_count * 0.0027

        #tout = MotorControl._ACK_TOUT_SEC
        if v > 1:
            retcode = self._modbus_transport.write_reg(
                self._motor_address, MotorControl._ANTICLOCKWISE + int(int(step_count / 256) * 512) + (step_count % 256), tout)
        else:
            retcode = self._modbus_transport.write_reg(
                self._motor_address, MotorControl._ANTICLOCKWISE + (step_count % 256), tout)
        return retcode

