import types

BASE_BOARD_V1_O = 0x0000
BASE_BOARD_V2_O = 0x0001
BASE_BOARD_V3_O = 0x0002
BASE_BOARD_V4_O = 0x0003

EXT_BOARD1_V1_O = 0x0004
EXT_BOARD1_V2_O = 0x0005
EXT_BOARD1_V3_O = 0x0006
EXT_BOARD1_V4_O = 0x0007

EXT_BOARD2_V1_O = 0x0008
EXT_BOARD2_V2_O = 0x0009
EXT_BOARD2_V3_O = 0x000A
EXT_BOARD2_V4_O = 0x000B

BASE_BOARD_USB1 = 0x000C
BASE_BOARD_USB2 = 0x000D


class PsuControl:
    _POWER_ON = 1
    _POWER_OFF = 0
    _ACK_TOUT_SEC = 1

    def __init__(self, modbus_transport, net_address):
        if net_address not in [BASE_BOARD_V1_O, BASE_BOARD_V2_O,
                               BASE_BOARD_V3_O, BASE_BOARD_V4_O,
                               EXT_BOARD1_V1_O, EXT_BOARD1_V2_O,
                               EXT_BOARD1_V3_O, EXT_BOARD1_V4_O,
                               EXT_BOARD2_V1_O, EXT_BOARD2_V2_O,
                               EXT_BOARD2_V3_O, EXT_BOARD2_V4_O,
                               BASE_BOARD_USB1, BASE_BOARD_USB2]:
            raise ValueError("Invalid _net_address")

        self._modbus_transport = modbus_transport
        self._net_address = net_address

    def turn_power_on(self):
        """
        :rtype: int
        """
        self._modbus_transport.write_bit(self._net_address, PsuControl._POWER_ON, PsuControl._ACK_TOUT_SEC)
        return types.RC_NONE

    def turn_power_off(self):
        """
        :rtype: int
        """
        self._modbus_transport.write_bit(self._net_address, PsuControl._POWER_OFF, PsuControl._ACK_TOUT_SEC)
        return types.RC_NONE


def read_psu_state(modbus_transport, start_net_address, net_count):
    """
    :param modbus_transport: types.ITransport
    :param start_net_address: int
    :param net_count: int
    :rtype: int
    """
    retcode = modbus_transport.read_bits(start_net_address, net_count)
    return retcode
