import time

import minimalmodbus

import types
from types import ITransport


class _ExtInstrument(minimalmodbus.Instrument):
    _MAX_BIT_COUNT = 14
    _NUMBER_OF_BYTES_BEFORE_REGISTERDATA = 1

    def __init__(self, end_point_address, slave_address):
        minimalmodbus.Instrument.__init__(self, end_point_address, slave_address)

    def read_bits(self, start_address, bit_count):
        functioncode = 0x01
        minimalmodbus._checkRegisteraddress(start_address)

        if bit_count > _ExtInstrument._MAX_BIT_COUNT:
            raise ValueError('Wrong bit_count > _MAX_BIT_COUNT')

        payload_to_slave = minimalmodbus._numToTwoByteString(start_address)
        payload_to_slave += minimalmodbus._numToTwoByteString(bit_count)

        payload_from_slave = self._performCommand(functioncode, payload_to_slave)
        registerdata = payload_from_slave[_ExtInstrument._NUMBER_OF_BYTES_BEFORE_REGISTERDATA:]
        retcode = minimalmodbus._unpack('H', registerdata)
        return retcode


class MinimalTransport(ITransport):
    _SLAVE_ADDRESS = 0x30
    _BAUDRATE = 19200
    _BYTESIZE = 8
    _STOPBITS = 1

    _WRITE_REGISTER_FN_CODE = 0x06
    _WRITE_BIT_FN_CODE = 0x05
    _RX_TO_TX_TOUT = 0.025

    def __init__(self, end_point_address):
        ITransport.__init__(self)

        self._ext_instrument = _ExtInstrument(end_point_address, MinimalTransport._SLAVE_ADDRESS)
        self._ext_instrument.serial.boudrate = MinimalTransport._BAUDRATE
        self._ext_instrument.serial.bytesize = MinimalTransport._BYTESIZE
        self._ext_instrument.serial.stopbits = MinimalTransport._STOPBITS
        # self._ext_instrument.debug = True

    def _open(self):
        self._ext_instrument.serial._close()
        self._ext_instrument.serial._open()

    def _close(self):
        self._ext_instrument.serial._close()

    def write_reg(self, address, int16_value, ack_tout):
        retcode = types.RC_FAIL
        try:
            self._open()
            self._ext_instrument.serial.timeout = ack_tout
            self._ext_instrument.write_register(address, int16_value,
                                                functioncode=MinimalTransport._WRITE_REGISTER_FN_CODE)
            self._close()
            time.sleep(MinimalTransport._RX_TO_TX_TOUT)
            retcode = types.RC_NONE
        except IOError:
            retcode = types.RC_EIO
        except (TypeError, ValueError):
            pass

        return retcode

    def write_bit(self, address, int1_value, ack_tout):
        retcode = types.RC_FAIL
        try:
            self._open()
            self._ext_instrument.serial.timeout = ack_tout
            self._ext_instrument.write_bit(address, int1_value,
                                           functioncode=MinimalTransport._WRITE_BIT_FN_CODE)
            self._close()
            time.sleep(MinimalTransport._RX_TO_TX_TOUT)
            retcode = types.RC_NONE
        except IOError:
            retcode = types.RC_EIO
        except (TypeError, ValueError):
            pass

        return retcode

    def read_bits(self, start_address, bit_count):
        reg_data = self._ext_instrument.read_bits(start_address, bit_count)
        return reg_data
