import time
from pymodbus.client.sync import ModbusSerialClient

import types
from types import ITransport


class PyTransport(ITransport):
    _SLAVE_ADDRESS = 0x30
    _BAUDRATE = 19200
    _BYTESIZE = 8
    _STOPBITS = 1
    _WRITE_BIT = 0x05
    _WRITE_REGISTER_FN_CODE = 0x06
    _RX_TO_TX_TOUT = 0.025

    def __init__(self, end_point_address):
        ITransport.__init__(self)
        self._modbus_serial_client = ModbusSerialClient(method="rtu")
        self._modbus_serial_client.baudrate = PyTransport._BAUDRATE
        self._modbus_serial_client.bytesize = PyTransport._BYTESIZE
        self._modbus_serial_client.port = end_point_address

    def _open(self):
        self._modbus_serial_client.close()
        self._modbus_serial_client.connect()

    def _close(self):
        self._modbus_serial_client.close()

    def read_reg(self, address, res, ack_tout):
        try:
            self._modbus_serial_client.timeout = ack_tout
            rc = self._modbus_serial_client.connect()
            v = 0
            if rc != 1:
                print "can't connect to device"
                res.append(types.RC_EIO)
                self._modbus_serial_client.close()
            else:
                rq = self._modbus_serial_client.read_input_registers(address, 1, unit=PyTransport._SLAVE_ADDRESS)
                if rq == None:
                    #print "timed out - no response received"
                    res.append(types.RC_FAIL)
                    self._modbus_serial_client.close()
                else:
                    # process 16 bits answer with reverse
                    if hasattr(rq, 'bits'):
                        v = int(''.join(reversed(map(lambda x: '1' if x else '0', rq.bits))),2)
                    elif hasattr(rq, 'registers'):
                        v = ((rq.registers[0] >> 8) & 0xFF) | ((rq.registers[0] << 8) & 0xFF00)
                    else:
                        print "response class format is not supported"
                    #print "return val :", v
                    self._modbus_serial_client.close()
                    time.sleep(PyTransport._RX_TO_TX_TOUT)
                    res.append(types.RC_NONE)
                    res.append(v)
        except IOError:  # TODO: PyModbus raises another types of exceptions.
            res.append(types.RC_EIO)
            print "read_input_registers: except IOError"
            self._modbus_serial_client.close()
        except (TypeError, ValueError):
            res.append(types.RC_FAIL)
            print "read_input_registers: error"
            self._modbus_serial_client.close()
            pass

    def write_reg(self, address, int16_value, ack_tout):
        retcode = types.RC_FAIL
        try:
            self._modbus_serial_client.timeout = ack_tout
            self._open()
            rr = self._modbus_serial_client.write_register(address, int16_value, unit=PyTransport._SLAVE_ADDRESS)
            print "write_register result: ", rr
            if rr == None:
                print "timed out - no response received"
            else:
                print "response: address - ", hex(rr.address), ", value - ", hex(rr.value)
            self._close()
            time.sleep(PyTransport._RX_TO_TX_TOUT)
            retcode = types.RC_NONE
        except IOError:  # TODO: PyModbus raises another types of exceptions.
            retcode = types.RC_EIO
            print "write_register: except IOError"
            self._close()
        except (TypeError, ValueError):
            print "write_register: error"
            self._close()
            pass

        return retcode

    def write_bit(self, address, int1_value, ack_tout):
        retcode = types.RC_FAIL
        try:
            self._modbus_serial_client.timeout = ack_tout
            self._open()
            rr = self._modbus_serial_client.write_coil(address, int1_value, unit=PyTransport._SLAVE_ADDRESS)
            print "write_coil result: ", rr
            if rr == None:
                print "timed out - no response received"
            else:
                print "response: "
            self._close()
            time.sleep(PyTransport._RX_TO_TX_TOUT)
            retcode = types.RC_NONE
        except IOError:
            retcode = types.RC_EIO
            print "write_coil: except IOError"
            self._close()
        except (TypeError, ValueError):
            print "write_coil: error"
            self._close()
            pass

        return retcode
