import time
import sys
from edge_psu_ctl import transport_pymodbus, motor_control
from edge.pipeline_helpers import get_psuctl_endpoint
from edge import eventlog

FULL_CW_ROTATE = 255
FULL_ACW_ROTATE = -255


def check_ver(com_port):
    modbus_instrument = transport_pymodbus.PyTransport(com_port)
    checkver = motor_control.CheckVer(modbus_instrument)
    res = []
    checkver.check_ver(res)
    if res[0] == 1:
        return res[1]
    else:
        return 0


def rotate_cw(com_port, motor_number, step, v):
    modbus_instrument = transport_pymodbus.PyTransport(com_port)
    motor = motor_control.MotorControl(modbus_instrument, motor_number, v)
    motor.rotate_cw(step, v)
    time.sleep(0.025)


def rotate_ccw(com_port, motor_number, step, v):
    modbus_instrument = transport_pymodbus.PyTransport(com_port)
    motor = motor_control.MotorControl(modbus_instrument, motor_number, v)
    motor.rotate_ccw(step, v)
    time.sleep(0.025)


def _main():
    raw_step = int(sys.argv[1])

    com_port = get_psuctl_endpoint()
    if not com_port:
        eventlog.log_d('fail: psu-ctl hardware not found')
        return

    v = check_ver(com_port)
    time.sleep(0.025)
    print "detected firmware version: ", hex(v)

    if v > 1:
        if raw_step < 0:
            rotate_ccw(com_port, motor_control.HEADER_1_4S2M__MOTOR2_FAST, -raw_step, v)

        elif raw_step > 0:
            rotate_cw(com_port, motor_control.HEADER_1_4S2M__MOTOR2_FAST, raw_step, v)

        else:
            pass

    else:
        if FULL_ACW_ROTATE < raw_step < 0:
            rotate_ccw(com_port, motor_control.HEADER_1_4S2M__MOTOR2, -raw_step, v)

        elif FULL_CW_ROTATE > raw_step > 0:
            rotate_cw(com_port, motor_control.HEADER_1_4S2M__MOTOR2, raw_step, v)

        elif raw_step >= FULL_CW_ROTATE:
            max_count = int(raw_step / FULL_CW_ROTATE)
            addition_step = int(raw_step % FULL_CW_ROTATE)
            count = 0
            while count < max_count:
                rotate_cw(com_port, motor_control.HEADER_1_4S2M__MOTOR2, FULL_CW_ROTATE, v)
                count += 1
            rotate_cw(com_port, motor_control.HEADER_1_4S2M__MOTOR2, addition_step, v)

        elif raw_step <= FULL_ACW_ROTATE:
            max_count = int(raw_step / FULL_ACW_ROTATE)
            addition_step = int(raw_step % FULL_ACW_ROTATE)
            count = 0
            while count < abs(max_count):
                rotate_ccw(com_port, motor_control.HEADER_1_4S2M__MOTOR2, FULL_CW_ROTATE, v)
                count += 1
            rotate_ccw(com_port, motor_control.HEADER_1_4S2M__MOTOR2, -addition_step, v)

        else:
            pass


if __name__ == '__main__':
    eventlog.open_log('rotate-focus', 'debug')
    # noinspection PyBroadException
    try:
        _main()
    except Exception:
        eventlog.log_x()
    finally:
        eventlog.close_log()