import sys
from edge_psu_ctl import transport_pymodbus, motor_control
from edge.pipeline_helpers import get_psuctl_endpoint
from edge import eventlog


def check_ver(com_port):
    modbus_instrument = transport_pymodbus.PyTransport(com_port)
    checkver = motor_control.CheckVer(modbus_instrument)
    res = []
    checkver.check_ver(res)
    if res[0] == 1:
        return res[1]
    else:
        return 0


def rotate_cw(com_port, motor_number, step, v):
    modbus_instrument = transport_pymodbus.PyTransport(com_port)
    motor = motor_control.MotorControl(modbus_instrument, motor_number, v)
    motor.rotate_cw(step, v)


def rotate_ccw(com_port, motor_number, step, v):
    modbus_instrument = transport_pymodbus.PyTransport(com_port)
    motor = motor_control.MotorControl(modbus_instrument, motor_number, v)
    motor.rotate_ccw(step, v)


def _main():
    raw_step = int(sys.argv[1])

    com_port = get_psuctl_endpoint()
    if not com_port:
        eventlog.log_d('fail: psu-ctl hardware not found')
        return

    v = check_ver(com_port)
    print "detected firmware version: ", hex(v)

    if raw_step < 0:
        if v > 1:
            rotate_ccw(com_port, motor_control.HEADER_1_4S2M__MOTOR1_FAST, -raw_step, v)
        else:
            rotate_ccw(com_port, motor_control.HEADER_1_4S2M__MOTOR1, -raw_step, v)
    elif raw_step > 0:
        if v > 1:
            rotate_cw(com_port, motor_control.HEADER_1_4S2M__MOTOR1_FAST, raw_step, v)
        else:
            rotate_cw(com_port, motor_control.HEADER_1_4S2M__MOTOR1, raw_step, v)
    else:
        pass


if __name__ == '__main__':
    eventlog.open_log('rotate-iris', 'debug')
    # noinspection PyBroadException
    try:
        _main()
    except Exception:
        eventlog.log_x()
    finally:
        eventlog.close_log()
